#!/bin/bash
######################################################################
#
#   tcar - The CentOS Artwork Repository
#   Copyright © 2014 The CentOS Artwork SIG
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#   Alain Reguera Delgado <alain.reguera@gmail.com>
#   39 Street No. 4426 Cienfuegos, Cuba.
#
######################################################################

function files_setConfigOption {

    local OPTION="${1}"

    case ${OPTION} in

        render-from )
            # Define value to "render-from" option. This information
            # is used to determine the source file that will be used
            # as input and must not be empty. This option can receive
            # more than one value by using several "render-from"
            # option definitions in the configuration file or one,
            # using path expansion.
            local FILE=''
            local FILES=$(tcarlib_getConfigValue)
            [[ -z ${FILES} ]] \
                && tcar_printMessage "`eval_gettext "The \\\"\\\$OPTION\\\" option cannot be empty."`" --as-error-line

            local COUNT=0

            for FILE in ${FILES};do
                if [[ ${FILE} =~ ^/ ]];then
                    RENDER_FROM[${COUNT}]=${FILE}
                else
                    RENDER_FROM[${COUNT}]=${RENDER_TARGET_DIR}/${FILE}
                fi
                COUNT=$(( ++COUNT ))
            done

            tcarlib_checkFiles -ef ${RENDER_FROM[*]}
            ;;

        render-type )
            # Define value to "render-type" option. This information
            # is used to determine the kind of processing that will be
            # performed. When the "render-type" option isn't provided
            # in the configuration file, use the file extension of the
            # first source file provided by "render-from" option as
            # value.
            RENDER_TYPE=$(tcarlib_getConfigValue)
            [[ -z ${RENDER_TYPE} ]] \
                && RENDER_TYPE=$(tcarlib_getFileExtension ${RENDER_FROM[0]})
            tcarlib_checkFiles -m '^[[:alnum:]]+$' ${RENDER_TYPE}
            ;;

        render-dir )
            # Define value of "render-dir" option. This information is
            # used to customize the directory where final files are
            # stored in.
            RENDER_TARGET_DIR=$(tcarlib_getConfigValue)
            [[ -z ${RENDER_TARGET_DIR} ]] && RENDER_TARGET_DIR=$(dirname ${CONFIGURATION})/Final
            ;;

        render-file )
            # Define value of "render-file" option. This information
            # is used to customize the path of final file being
            # produced.
            RENDER_TARGET=$(tcarlib_getConfigValue)
            if [[ ${RENDER_TARGET} =~ ^/ ]];then
                RENDER_TARGET=${RENDER_TARGET}
            else
                RENDER_TARGET=${RENDER_TARGET_DIR}/${SECTION}
            fi
            ;;

        render-logic )
            # Define value of "render-logic" option. This information
            # is used to determined whether to remove or not the
            # directory where final content is stored in, before new
            # content be put inside it.
            RENDER_LOGIC=$(tcarlib_getConfigValue)
            if [[ ${RENDER_LOGIC} == 'remove-first' ]];then
                if [[ -d ${RENDER_TARGET_DIR} ]];then
                    rm -r ${RENDER_TARGET_DIR}
                    tcarlib_setParentDir "${RENDER_TARGET}"
                fi
            else
                RENDER_LOGIC='overwrite'
            fi
            ;;

        locale-from )
            # Define value to "locale-from" option. This information
            # is used to retrieve the translation file -in PO format-
            # associated to the source files provided through
            # "render-from" option. This option can be empty -to
            # specify no translation available- or receive one or more
            # values. The order in which translation files are
            # provided here must be the same provided in render-from
            # to specify source files. So, the relation source
            # file-translation file can be accurate.
            local FILE=''
            local FILES=$(tcarlib_getConfigValue)

            local COUNT=0

            [[ -z ${FILES} ]] \
                && LOCALE_FROM[${COUNT}]=${FILE} \
                && return

            for FILE in ${FILES};do
                if [[ ${FILE} =~ ^/ ]];then
                    LOCALE_FROM[${COUNT}]=${FILE}
                else
                    LOCALE_FROM[${COUNT}]=${RENDER_TARGET_DIR}/${FILE}
                fi
                COUNT=$(( ++COUNT ))
            done

            # Don't verify existence of translation files here. They
            # are optional.

            # Verify whether the design model has translation file or
            # not and, redefine the target file based on it.
            if [[ ${#LOCALE_FROM[*]} -gt 0 ]];then
                RENDER_TARGET_DIR="${RENDER_TARGET_DIR}/${SCRIPT_LANG_LC}"
                RENDER_TARGET=''      ; files_setConfigOption 'render-file'
            fi
            ;;

        release )
            # Define value to "release" option. This information is
            # used by translation markers when source files are
            # processed. When this option isn't provided, it is
            # determined from the current operating system.
            RELEASE=$(tcarlib_getConfigValue)
            [[ -z ${RELEASE} ]] && RELEASE=$(tcarlib_printSystemRelease)
            ;;

        release-major )
            MAJOR_RELEASE=$(tcarlib_getConfigValue)
            [[ -z ${MAJOR_RELEASE} ]] && MAJOR_RELEASE=$(echo ${RELEASE} | cut -d. -f1)
            ;;

        command )
            COMMAND=$(tcarlib_getConfigValue)
            [[ -z ${COMMAND} ]] && COMMAND='/bin/cp'
            ;;

        * )
            tcarlib_printMessage "`eval_gettext "The \\\"\\\$OPTION\\\" option isn't supported."`" --as-error-line
            ;;

    esac

}
