#!/bin/bash
######################################################################
#
#   tcar - The CentOS Artwork Repository
#   Copyright © 2014 The CentOS Artwork SIG
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#   Alain Reguera Delgado <alain.reguera@gmail.com>
#   39 Street No. 4426 Cienfuegos, Cuba.
#
######################################################################

# Standardize construction of translation markers and their related
# expansion. As convention, translation markers must be set inside
# source files (e.g., Docbook, Svg, etc.) and expanded inside temporal
# instances used to produce final contents.
function tcar_setTranslationMarkers {

    # Initialize variables.
    local -a SRC
    local -a DST
    local COUNT=0
    local COUNTSRC=0
    local COUNTDST=0

    # Define source location on which sed replacements take place.
    local LOCATION="${1}"

    # Verify that source location does exist.
    tcarlib_checkFiles -ef ${LOCATION}

    # Define copyright translation markers.
    SRC[((++${#SRC[*]}))]='=COPYRIGHT_YEAR_FIRST='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --first-year)"
    SRC[((++${#SRC[*]}))]='=COPYRIGHT_YEAR(_LAST)?='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --year)"
    SRC[((++${#SRC[*]}))]='=COPYRIGHT_YEAR(S)?_LIST='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --years-list)"
    SRC[((++${#SRC[*]}))]='=COPYRIGHT_YEAR(S)?_RANGE='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --years-range)"
    SRC[((++${#SRC[*]}))]='=COPYRIGHT_HOLDER='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --holder)"
    SRC[((++${#SRC[*]}))]='=COPYRIGHT_HOLDER_PREDICATE='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --holder-predicate)"

    # Define license translation markers.
    SRC[((++${#SRC[*]}))]='=LICENSE='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --license)"
    SRC[((++${#SRC[*]}))]='=LICENSE_URL='
    DST[((++${#DST[*]}))]="$(tcar_printCopyright --license-url)"

    # Define theme translation markers.
    SRC[((++${#SRC[*]}))]='=THEME='
    DST[((++${#DST[*]}))]="$(tcar_printPathComponent ${LOCATION} --motif)"

    # Define url translation markers.
    SRC[((++${#SRC[*]}))]='=URL='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'home')
    SRC[((++${#SRC[*]}))]='=URL_WIKI='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'wiki')
    SRC[((++${#SRC[*]}))]='=URL_LISTS='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'lists')
    SRC[((++${#SRC[*]}))]='=URL_FORUMS='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'forums')
    SRC[((++${#SRC[*]}))]='=URL_MIRRORS='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'mirrors')
    SRC[((++${#SRC[*]}))]='=URL_DOCS='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'documentation')
    SRC[((++${#SRC[*]}))]='=URL_BUGS='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'bugs')
    SRC[((++${#SRC[*]}))]='=URL_GIT='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'git')
    SRC[((++${#SRC[*]}))]='=URL_PLANET='
    DST[((++${#DST[*]}))]=$(tcarlib_printUrl 'planet')

    # Define release translation markers.
    SRC[((++${#SRC[*]}))]='=RELEASE='
    DST[((++${#DST[*]}))]=${RELEASE}
    SRC[((++${#SRC[*]}))]='=MAJOR_RELEASE='
    DST[((++${#DST[*]}))]=${MAJOR_RELEASE}

    # Define emails translation markers.
    SRC[((++${#SRC[*]}))]='=MAIL_DOCS='
    DST[((++${#DST[*]}))]="$(tcarlib_printMail 'documentation')"

    # Define design models translation markers.
    SRC[((++${#SRC[*]}))]='=BACKGROUND_IMAGE='
    DST[((++${#DST[*]}))]="${RENDER_BACK}"

    # Define locale translation markers.
    SRC[((++${#SRC[*]}))]='=LOCALE='
    DST[((++${#DST[*]}))]="${SCRIPT_LANG_LC}"
    SRC[((++${#SRC[*]}))]='=LOCALE_LL='
    DST[((++${#DST[*]}))]="${SCRIPT_LANG_LL}"
    SRC[((++${#SRC[*]}))]='=LOCALE_CC='
    DST[((++${#DST[*]}))]="${SCRIPT_LANG_CC}"

    # Define repository translation markers.
    SRC[((++${#SRC[*]}))]='=(REPO_TLDIR|REPO_HOME|TCAR_BASEDIR)='
    DST[((++${#DST[*]}))]="${TCAR_BASEDIR}"

    # Define repository workplace translation markers.
    SRC[((++${#SRC[*]}))]='=TCAR_WORKDIR='
    DST[((++${#DST[*]}))]="${TCAR_WORKDIR}"

    # Do replacement of nested translation markers.
    while [[ ${COUNTDST} -lt ${#DST[@]} ]];do

        # Verify existence of translation markers. If there is no
        # translation marker on replacement, continue with the next
        # one in the list.
        if [[ ! ${DST[${COUNTDST}]} =~ =[[:upper:]_]+= ]];then
            # Increment destination counter.
            COUNTDST=$((${COUNTDST} + 1))
            # The current replacement value doesn't have translation
            # marker inside, so skip it and evaluate the next
            # replacement value in the list.
            continue
        fi

        while [[ ${COUNTSRC} -lt ${#SRC[*]} ]];do

            # Update replacements.
            DST[${COUNTDST}]=$(echo ${DST[${COUNTDST}]} \
                | sed -r "s!${SRC[${COUNTSRC}]}!${DST[${COUNTSRC}]}!g")

            # Increment source counter.
            COUNTSRC=$((${COUNTSRC} + 1))

        done

        # Reset source counter
        COUNTSRC=0

        # Increment destination counter.
        COUNTDST=$((${COUNTDST} + 1))

    done

    # Apply replacements for translation markers.
    while [[ ${COUNT} -lt ${#SRC[*]} ]];do

        # Use sed to replace translation markers inside the design
        # model instance.
        sed -r -i "s!${SRC[${COUNT}]}!${DST[${COUNT}]}!g" ${LOCATION}

        # Increment counter.
        COUNT=$((${COUNT} + 1))

    done

    # Remove escaped character from translation markers. This is one
    # of the reasons why translation marker should be expanded in
    # source files instances not the source files themselves.
    # Escaping translation markers provides a way of talking about
    # them without expanding them.
    sed -r -i 's/(=)\\([[:upper:]_]+=)/\1\2/g' ${LOCATION}

    # Unset specific translation markers and specific replacement
    # variables in order to clean them up. Otherwise, undesired values
    # may remain from one file to another.
    unset SRC
    unset DST

}
